{
*****************************************************************************
*  Name    : PWM.BAS                                                        *
*  Author  : David John Barker                                              *
*  Notice  : Copyright (c) 2007 Mecanique                                   *
*          : All Rights Reserved                                            *
*  Date    : 23/08/2007                                                     *
*  Version : 1.0                                                            *
*  Notes   : From an idea found at http://www.eng-serve.net/pic             *                    
*          :                                                                *
*****************************************************************************
}
Module PWM
Dim
   FMaxDuty As Word,
   FTMR2ON As T2CON.2

#if _device in (18F1220, 18F1320)
Dim FPWMPin As PORTB.3
#else
Dim FPWMPin As PORTC.2
#endif
{
****************************************************************************
* Name    : Start                                                          *
* Purpose :                                                                *
****************************************************************************
}  
Public Sub Start()
    CCP1CON = $0C
    Output(FPWMPin) 
    FTMR2ON = 1   
End Sub
{
****************************************************************************
* Name    : Stop                                                           *
* Purpose :                                                                *
****************************************************************************
}  
Public Sub Stop()
   Input(FPWMPin)
   CCP1CON = $00
End Sub
{
****************************************************************************
* Name    : SetDuty                                                        *
* Purpose : The CCPR1L contains the eight MSbs And the CCP1CON<5:4>        *
*         : contains the two LSbs. This 10-Bit value is represented by     *
*         : CCPR1L:CCP1CON<5:4>.                                           *
****************************************************************************
} 	
Public Sub SetDuty(pDuty As Word)
   CCP1CON.5 = pDuty.1
   CCP1CON.4 = pDuty.0	
   CCPR1L = pDuty >> 2
End Sub
{
****************************************************************************
* Name    : SetDutyPercent                                                 *
* Purpose : Set the duty as a percentage                                   *
****************************************************************************
} 	
Public Sub SetDutyPercent(pPercent As Byte)
   SetDuty(FMaxDuty * pPercent / 100)
End Sub
{
****************************************************************************
* Name    : MaxDuty                                                        *
* Purpose :                                                                *
****************************************************************************
}   
Public Inline Function MaxDuty() As FMaxDuty
End Function
{
****************************************************************************
* Name    : SetFreq                                                        *
* Purpose :                                                                *
****************************************************************************
} 
Public Function SetFreq(pFrequency As Word, pInitialDuty As Byte = 0) As Boolean
   Const Fosc As LongWord = _clock * 1000000
   Dim Prescale As Byte
   Dim PR2Value, PRConst As Word

   // loop through all the valid prescalers...
   PRConst = Fosc / pFrequency / 4
   Prescale = 1
   Result = false
   Repeat
      PR2Value = PRConst / Prescale - 1            // calculate a PR2 value
      If (PR2Value < 256) And (PR2Value > 1) Then  // if it is a valid value, then...
         Result = true                             // function return true (success)
         Select Prescale                           // configure T2CON prescale
            Case 1  : T2CON = %00000000            // prescale 1
	        Case 4  : T2CON = %00000001            // prescale 4
	        Case 16 : T2CON = %00000011            // prescale 16
         End Select
	     PR2 = PR2Value                            // initialise PR2
         FMaxDuty = Fosc / (pFrequency * Prescale) // determine maximum duty...
         SetDutyPercent(pInitialDuty)              // set to initial duty
         Start                                     // start PWM
         Exit                                      // exit the sub
      EndIf   
      Prescale = Prescale * 4
   Until Prescale > 16
End Function

// initialise the module
FMaxDuty = 0
